@@ -1,4 +1,4 @@
-﻿# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+﻿# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -404,6 +404,21 @@ class ImageGenerator(object):
         self.save(s, "main_menu", overwrite=False)
         self.save(s, "game_menu", overwrite=False)
 
+    def generate_bubble(self):
+
+        import shutil
+
+        for fn in [ "bubble.png", "thoughtbubble.png" ]:
+
+            source = os.path.join(config.renpy_base, "gui", "game", "gui", fn)
+            dest = os.path.join(self.prefix, fn)
+
+            if source == dest:
+                return
+
+            shutil.copyfile(source, dest)
+
+
     def generate_all(self):
         self.generate_textbox()
         self.generate_choice_button()
@@ -415,6 +430,7 @@ class ImageGenerator(object):
         self.generate_notify()
         self.generate_menus()
         self.generate_icon()
+        self.generate_bubble()
 
 
 if __name__ == "__main__":
