@@ -51,9 +51,9 @@ void pixellate32_core(PyObject *pysrc,
     SDL_Surface *dst;
 
     int x, y, i, j;
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int srcw, srch;
+    int dstw, dsth;
     int vw, vh;
 
     unsigned char *srcpixels;
@@ -193,9 +193,9 @@ void pixellate24_core(PyObject *pysrc,
     SDL_Surface *dst;
 
     int x, y, i, j;
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int srcw, srch;
+    int dstw, dsth;
     int vw, vh;
 
     unsigned char *srcpixels;
@@ -322,9 +322,8 @@ void map32_core(PyObject *pysrc,
     SDL_Surface *dst;
 
     int x, y;
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int srcw, srch;
 
     char *srcpixels;
     char *dstpixels;
@@ -344,9 +343,7 @@ void map32_core(PyObject *pysrc,
     srcpitch = src->pitch;
     dstpitch = dst->pitch;
     srcw = src->w;
-    dstw = dst->w;
     srch = src->h;
-    dsth = dst->h;
 
     srcrow = srcpixels;
     dstrow = dstpixels;
@@ -381,9 +378,8 @@ void map24_core(PyObject *pysrc,
     SDL_Surface *dst;
 
     int x, y;
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int srcw, srch;
 
     char *srcpixels;
     char *dstpixels;
@@ -403,9 +399,7 @@ void map24_core(PyObject *pysrc,
     srcpitch = src->pitch;
     dstpitch = dst->pitch;
     srcw = src->w;
-    dstw = dst->w;
     srch = src->h;
-    dsth = dst->h;
 
     srcrow = srcpixels;
     dstrow = dstpixels;
@@ -446,9 +440,8 @@ void linmap32_core(PyObject *pysrc,
     SDL_Surface *dst;
 
     int x, y;
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int srcw, srch;
 
     char *srcpixels;
     char *dstpixels;
@@ -468,9 +461,7 @@ void linmap32_core(PyObject *pysrc,
     srcpitch = src->pitch;
     dstpitch = dst->pitch;
     srcw = src->w;
-    dstw = dst->w;
     srch = src->h;
-    dsth = dst->h;
 
     srcrow = srcpixels;
     dstrow = dstpixels;
@@ -505,9 +496,8 @@ void linmap24_core(PyObject *pysrc,
     SDL_Surface *dst;
 
     int x, y;
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int srcw, srch;
 
     char *srcpixels;
     char *dstpixels;
@@ -527,9 +517,7 @@ void linmap24_core(PyObject *pysrc,
     srcpitch = src->pitch;
     dstpitch = dst->pitch;
     srcw = src->w;
-    dstw = dst->w;
     srch = src->h;
-    dsth = dst->h;
 
     srcrow = srcpixels;
     dstrow = dstpixels;
@@ -653,8 +641,8 @@ void linblur32_core(PyObject *pysrc,
     SDL_Surface *src;
     SDL_Surface *dst;
 
-    Uint32 rows, cols;
-    Uint32 incr, skip;
+    int rows, cols;
+    int incr, skip;
 
     unsigned char *srcpixels;
     unsigned char *dstpixels;
@@ -793,8 +781,8 @@ void linblur24_core(PyObject *pysrc,
     SDL_Surface *src;
     SDL_Surface *dst;
 
-    Uint32 rows, cols;
-    Uint32 incr, skip;
+    int rows, cols;
+    int incr, skip;
 
     unsigned char *srcpixels;
     unsigned char *dstpixels;
@@ -928,9 +916,8 @@ void alphamunge_core(PyObject *pysrc,
     SDL_Surface *src;
     SDL_Surface *dst;
 
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int dstw, dsth;
 
     unsigned char *srcpixels;
     unsigned char *dstpixels;
@@ -951,9 +938,7 @@ void alphamunge_core(PyObject *pysrc,
     dstpixels = (unsigned char *) dst->pixels;
     srcpitch = src->pitch;
     dstpitch = dst->pitch;
-    srcw = src->w;
     dstw = dst->w;
-    srch = src->h;
     dsth = dst->h;
 
 
@@ -996,9 +981,8 @@ void scale32_core(PyObject *pysrc, PyObject *pydst,
     SDL_Surface *dst;
 
     int y;
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int dstw, dsth;
     float xdelta, ydelta;
 
     unsigned char *srcpixels;
@@ -1014,9 +998,7 @@ void scale32_core(PyObject *pysrc, PyObject *pydst,
     dstpixels = (unsigned char *) dst->pixels;
     srcpitch = src->pitch;
     dstpitch = dst->pitch;
-    srcw = src->w;
     dstw = dst->w;
-    srch = src->h;
     dsth = dst->h;
 
     if (precise) {
@@ -1041,7 +1023,6 @@ void scale32_core(PyObject *pysrc, PyObject *pydst,
     for (y = 0; y < dsth; y++) {
 
         unsigned char *s0;
-        unsigned char *s1;
         unsigned char *d;
         unsigned char *dend;
 
@@ -1058,7 +1039,6 @@ void scale32_core(PyObject *pysrc, PyObject *pydst,
         s0frac = 256 - s1frac;
 
         s0 = srcpixels + (sline >> 8) * srcpitch;
-        s1 = s0 + srcpitch;
 
         scol = source_xoff * 256 + dest_xoff * xdelta;
 
@@ -1109,15 +1089,13 @@ void scale24_core(PyObject *pysrc, PyObject *pydst,
     SDL_Surface *dst;
 
     int y;
-    Uint32 srcpitch, dstpitch;
-    Uint32 srcw, srch;
-    Uint32 dstw, dsth;
+    int srcpitch, dstpitch;
+    int dstw, dsth;
     float xdelta, ydelta;
 
     unsigned char *srcpixels;
     unsigned char *dstpixels;
 
-
     src = PySurface_AsSurface(pysrc);
     dst = PySurface_AsSurface(pydst);
 
@@ -1127,9 +1105,7 @@ void scale24_core(PyObject *pysrc, PyObject *pydst,
     dstpixels = (unsigned char *) dst->pixels;
     srcpitch = src->pitch;
     dstpitch = dst->pitch;
-    srcw = src->w;
     dstw = dst->w;
-    srch = src->h;
     dsth = dst->h;
 
     xdelta = 255.0 * (source_width - 1) / dest_width;
@@ -1138,7 +1114,6 @@ void scale24_core(PyObject *pysrc, PyObject *pydst,
     for (y = 0; y < dsth; y++) {
 
         unsigned char *s0;
-        unsigned char *s1;
         unsigned char *d;
         unsigned char *dend;
 
@@ -1155,7 +1130,6 @@ void scale24_core(PyObject *pysrc, PyObject *pydst,
         s0frac = 256 - s1frac;
 
         s0 = srcpixels + (sline >> 8) * srcpitch;
-        s1 = s0 + srcpitch;
 
         scol = source_xoff * 255 + dest_xoff * xdelta;
 
