@@ -1,4 +1,4 @@
-# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -38,6 +38,64 @@ TAG_RE = re.compile(r'(\{\{)|(\{(p|w|nw|fast|done)(?:\=([^}]*))?\})', re.S)
 less_pauses = ("RENPY_LESS_PAUSES" in os.environ)
 
 
+class Callbacks(object):
+    """
+    This stores and calls the character callbacks.
+    """
+
+    def __init__(self, callbacks, interact, type, cb_args, multiple):
+        self.callbacks = callbacks
+        self.interact = interact
+        self.type = type
+        self.cb_args = cb_args
+        self.multiple = multiple # type: tuple[int, int]|None
+
+        self.what = None # type: str|None
+        self.start = None # type: int|None
+        self.end = None # type: int|None
+        self.delay = None # type: float|None
+        self.last_segment = None # type: bool|None
+
+    def __call__(self, *args, **kwargs):
+        if not self.callbacks:
+            return
+
+        kwargs["type"] = self.type
+        kwargs["interact"] = self.interact
+
+        if renpy.config.character_callback_compat is None:
+
+            if self.what is not None:
+                kwargs["what"] = self.what
+
+            if self.start is not None:
+                kwargs["start"] = self.start
+
+            if self.end is not None:
+                kwargs["end"] = self.end
+
+            if self.delay is not None:
+                kwargs["delay"] = self.delay
+
+            if self.last_segment is not None:
+                kwargs["last_segment"] = self.last_segment
+
+            if  self.multiple is not None:
+                kwargs["multiple"] = self.multiple
+
+            kwargs["please_ignore_unknown_keyword_arguments"] = None
+
+        kwargs.update(self.cb_args)
+
+        for c in self.callbacks:
+            c(*args, **kwargs)
+
+    def copy(self):
+        rv = Callbacks(self.callbacks, self.interact, self.type, self.cb_args, self.multiple)
+        rv.__dict__.update(self.__dict__)
+        return rv
+
+
 class DialogueTextTags(object):
     """
     This object parses the text tags that only make sense in dialogue,
@@ -95,6 +153,11 @@ class DialogueTextTags(object):
                 elif tag == "nw":
                     self.no_wait = True
 
+                    if value is not None and not less_pauses:
+                        self.pause_start.append(len(self.text))
+                        self.pause_end.append(len(self.text))
+                        self.pause_delay.append(value)
+
                 elif tag == "fast":
                     self.pause_start = [ len(self.text) ]
                     self.pause_end = [ ]
@@ -240,6 +303,7 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
                      layer=None,
                      properties={},
                      multiple=None,
+                     retain=None,
                      **kwargs):
     """
     This is called (by default) by renpy.display_say to add the
@@ -270,6 +334,11 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
 
     @param kwargs: Additional keyword arguments should be ignored.
 
+    `retain`
+        If not None, the screen should be retained (not transient),
+        and the screen should be given the value of this argument as
+        its tag.
+
     This function is required to return the ui.text() widget
     displaying the what text.
     """
@@ -305,7 +374,10 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
 
         tag = screen
 
-        if multiple:
+        if retain:
+            tag = retain
+
+        elif multiple:
 
             if renpy.display.screen.has_screen("multiple_" + screen):
                 screen = "multiple_" + screen
@@ -325,7 +397,7 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
         renpy.display.screen.show_screen(
             screen,
             _widget_properties=props,
-            _transient=True,
+            _transient=not retain,
             _tag=tag,
             who=who,
             what=what,
@@ -412,8 +484,7 @@ class SlowDone(object):
             renpy.ui.pausebehavior(self.delay, True, voice=self.last_pause and not self.no_wait, self_voicing=self.last_pause)
             renpy.exports.restart_interaction()
 
-        for c in self.callback:
-            c("slow_done", interact=self.interact, type=self.type, **self.cb_args)
+        self.callback("slow_done")
 
 # This is a queue for text that's going to be passed to the say behavior to
 # set the AFM info.
@@ -439,7 +510,8 @@ def display_say(
         ctc_force=False,
         advance=True,
         multiple=None,
-        dtt=None):
+        dtt=None,
+        retain=False):
 
     global afm_text_queue
 
@@ -477,6 +549,9 @@ def display_say(
 
         # Clears out transients.
         renpy.exports.with_statement(None)
+
+        renpy.exports.checkpoint(True, hard=checkpoint)
+
         return
 
     # If we're not interacting, call the noniteractive_callbacks.
@@ -495,10 +570,7 @@ def display_say(
         callback = [ callback ]
 
     callback = renpy.config.all_character_callbacks + callback
-
-    # Call the begin callback.
-    for c in callback:
-        c("begin", interact=interact, type=type, **cb_args)
+    callback = Callbacks(callback, interact, type, cb_args, multiple)
 
     roll_forward = renpy.exports.roll_forward_info()
 
@@ -538,6 +610,22 @@ def display_say(
 
     exception = None
 
+    retain_tag = "_retain_0"
+    retain_count = -1
+
+    if retain:
+
+        while True:
+            retain_count += 1
+            retain_tag = "_retain_{}".format(retain_count)
+
+            if not renpy.exports.get_screen(retain_tag):
+                break
+
+    # Call the begin callback.
+    callback.what = dtt.text
+    callback("begin")
+
     if dtt.fast:
         for i in renpy.config.say_sustain_callbacks:
             i()
@@ -549,6 +637,13 @@ def display_say(
             # True if the is the last pause in a line of dialogue.
             last_pause = (i == len(pause_start) - 1)
 
+            # Create a callback object specific to this pause.
+            pause_callback = callback.copy()
+            pause_callback.start = start
+            pause_callback.end = end
+            pause_callback.delay = delay
+            pause_callback.last_segment = last_pause
+
             # If we're going to do an interaction, then saybehavior needs
             # to be here.
             if advance:
@@ -597,11 +692,10 @@ def display_say(
                 ctc = None
 
             # Run the show callback.
-            for c in callback:
-                c("show", interact=interact, type=type, **cb_args)
+            pause_callback("show")
 
             # Create the callback that is called when the slow text is done.
-            slow_done = SlowDone(what_ctc, ctc_position, callback, interact, type, cb_args, delay, ctc_kwargs, last_pause, dtt.no_wait)
+            slow_done = SlowDone(what_ctc, ctc_position, pause_callback, interact, type, cb_args, delay, ctc_kwargs, last_pause, dtt.no_wait)
 
             extend_text = ""
 
@@ -629,12 +723,29 @@ def display_say(
                     extend_text = "{done}" + extend_text
 
             # Show the text.
-            if multiple:
-                what_text = show_function(who, what_string, multiple=multiple)
-            else:
-                what_text = show_function(who, what_string)
 
+            show_args = { }
+
+            if multiple:
+                show_args["multiple"] = multiple
+
+            if retain:
+                show_args["retain"] = retain_tag
+
+            what_text = show_function(who, what_string, **show_args)
+
+            # What text is (screen, id, layer) tuple if we're using a screen.
             if isinstance(what_text, tuple):
+                # If this is not the first pause, set the transform event to "replace".
+                if i != 0 and renpy.config.say_replace_event:
+                    screen_displayable = renpy.display.screen.get_screen(what_text[0], what_text[2])
+                    if screen_displayable is not None:
+                        screen_displayable.set_transform_event("replace")
+
+                        if not retain and not last_pause:
+                            sls = renpy.game.context().scene_lists
+                            sls.set_transient_prefix(what_text[2], what_text[0], "replaced")
+
                 what_text = renpy.display.screen.get_widget(what_text[0], what_text[1], what_text[2])
 
             if not multiple:
@@ -683,8 +794,7 @@ def display_say(
 
                 slow = False
 
-            for c in callback:
-                c("show_done", interact=interact, type=type, **cb_args)
+            pause_callback("show_done")
 
             if not slow:
                 slow_done()
@@ -730,8 +840,7 @@ def display_say(
 
         renpy.plog(1, "after with none")
 
-    for c in callback:
-        c("end", interact=interact, type=type, **cb_args)
+    callback("end")
 
     if exception is not None:
         raise exception
@@ -859,6 +968,7 @@ class ADVCharacter(object):
             callback=d('callback'),
             type=d('type'),
             advance=d('advance'),
+            retain=d('retain'),
             )
 
         self._statement_name = properties.pop("statement_name", None)
@@ -959,34 +1069,27 @@ class ADVCharacter(object):
         return screen, show_args, who_args, what_args, window_args, properties
 
 
-    def do_show(self, who, what, multiple=None, extra_properties=None):
+    def do_show(self, who, what, multiple=None, extra_properties=None, retain=None):
 
         screen, show_args, who_args, what_args, window_args, properties = self.get_show_properties(extra_properties)
 
+        show_args = dict(show_args)
+
         if multiple is not None:
+            show_args["multiple"] = multiple
 
-            return self.show_function(
-                who,
-                what,
-                who_args=who_args,
-                what_args=what_args,
-                window_args=window_args,
-                screen=screen,
-                properties=properties,
-                multiple=multiple,
-                **show_args)
+        if retain:
+            show_args["retain"] = retain
 
-        else:
-
-            return self.show_function(
-                who,
-                what,
-                who_args=who_args,
-                what_args=what_args,
-                window_args=window_args,
-                screen=screen,
-                properties=properties,
-                **show_args)
+        return self.show_function(
+            who,
+            what,
+            who_args=who_args,
+            what_args=what_args,
+            window_args=window_args,
+            screen=screen,
+            properties=properties,
+            **show_args)
 
 
     # This is called after the last interaction is done.
@@ -1089,7 +1192,7 @@ class ADVCharacter(object):
         renpy.game.context().say_attributes = None
 
         temporary_attrs = renpy.game.context().temporary_attributes
-        renpy.game.context().say_attributes = None
+        renpy.game.context().temporary_attributes = None
 
         if interact:
             if temporary_attrs:
@@ -1588,10 +1691,6 @@ def Character(name=NotSet, kind=None, **properties):
         unable to move past the say statement unless an alternate means
         (such as a jump hyperlink or screen) is provided.
 
-    `mode`
-        A string giving the mode to enter when this character
-        speaks. See the section on :ref:`modes <modes>` for more details.
-
     `callback`
         A function that is called when events occur while the
         character is speaking. See the section on
@@ -1631,6 +1730,12 @@ def Character(name=NotSet, kind=None, **properties):
     `screen`
         The name of the screen that is used to display the dialogue.
 
+    `retain`
+        If not true, an unused tag is generated for each line of dialogue,
+        and the screens are shown non-transiently. Call :func:`renpy.clear_retain`
+        to remove all retaint screens. This is almost always used with
+        :doc:`bubble`.
+
     Keyword arguments beginning with ``show_`` have the prefix
     stripped off, and are passed to the screen as arguments. For
     example, the value of ``show_myflag`` will become the value of
