@@ -1,4 +1,4 @@
-# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -142,11 +142,11 @@ from future.builtins import chr
 def add_attribute(obj, name, value):
     pass
 
-try:
-    if PY2:
+if PY2:
+    try:
         from renpy.compat.dictviews import add_attribute
-except ImportError:
-    print("Could not import renpy.compat.dictviews.", file=sys.stderr)
+    except ImportError:
+        print("Could not import renpy.compat.dictviews.", file=sys.stderr)
 
 
 ################################################################################
@@ -192,6 +192,16 @@ if PY2:
 
         subprocess.Popen = Popen
 
+################################################################################
+# Intern
+
+if PY2:
+    intern_cache = {}
+
+    def intern(s):
+        return intern_cache.setdefault(s, s)
+
+    sys.intern = intern
 
 ################################################################################
 # Export functions.
@@ -205,6 +215,5 @@ if PY2:
 
 # Generated by scripts/relative_imports.py, do not edit below this line.
 if 1 == 0:
-    from . import dictviews
     from . import fixes
     from . import pickle
