@@ -1,4 +1,4 @@
-# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -33,6 +33,7 @@ from renpy.display.render import render, Render, redraw
 from renpy.display.core import absolute
 from renpy.display.behavior import map_event, run, run_unhovered
 
+import weakref
 
 def default_drag_group():
     """
@@ -58,7 +59,7 @@ def default_drop_allowable(drop, drags):
     return True
 
 
-class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
+class Drag(renpy.display.displayable.Displayable, renpy.revertable.RevertableObject):
     """
     :doc: drag_drop class
     :args: (d=None, drag_name=None, draggable=True, droppable=True, drag_raise=True, dragging=None, dragged=None, dropped=None, drag_handle=(0.0, 0.0, 1.0, 1.0), drag_joined=..., clicked=None, hovered=None, unhovered=None, mouse_drop=False, **properties)
@@ -93,6 +94,9 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
     has been computed, the layout properties are ignored in favor of the
     position stored inside the Drag.
 
+    Transforms should not be applied to a Drag directly. Instead, apply
+    the transform to the child of the Drag.
+
     `d`
         If present, the child of this Drag. Drags use the child style
         in preference to this, if it's not None.
@@ -117,7 +121,7 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
     `activated`
         A callback (or list of callbacks) that is called when the mouse
         is pressed down on the drag. It is called with one argument, a
-        a list of Drags that are being dragged. The return value of this
+        list of Drags that are being dragged. The return value of this
         callback is ignored.
 
     `dragging`
@@ -146,10 +150,10 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
         None.
 
     `clicked`
-        A callback this is called, with no arguments, when the Drag is
-        clicked without being moved. A droppable can also be focused
-        and clicked.  If the callback returns a value other than None,
-        that value is returned as the result of the interaction.
+        A callback that is called when the Drag is clicked without being moved.
+        It is called with one argument, the Drag being clicked on. A droppable
+        can also be focused and clicked. If the callback returns a value other
+        than None, that value is returned as the result of the interaction.
 
     `alternate`
         An action that is run when the Drag is right-clicked (on the
@@ -157,18 +161,26 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
         be necessary to increase :var:`config.longpress_duration` if
         this triggers to early on mobile platforms.
 
+    `hovered`
+        An action to run when the drag gains focus.
+
+    `unhovered`
+        An action to run when the drag loses focus.
+
+    `tooltip`
+        A tooltip to display when the drag is hovered over.
+
     `drag_handle`
         A (x, y, width, height) tuple, giving the position of the drag
-        handle within the child. In this tuple, integers are considered
-        to be a literal number of pixels, while floats are relative to
-        the size of the child.
+        handle within the child. This tuple takes :term:`positions <position>`.
 
     `drag_joined`
         This is called with the current Drag as an argument. It's
         expected to return a list of [ (drag, x, y) ] tuples, giving
         the draggables to drag as a unit. `x` and `y` are the offsets
         of the drags relative to each other, they are not relative
-        to the corner of this drag.
+        to the corner of this drag. `drag` is either the Drag object
+        to be joined or the drag_name of such a Drag.
 
     `drag_offscreen`
         Determines the conditions under which the drag is allowed
@@ -255,6 +267,23 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
         The first is the Drag which determines its sensitivity.
         The second is a list of Drags that are being dragged.
 
+    `snapped`
+        A callback (or list of callbacks) that is called when the Drag completes
+        a snap animation. It is called with four arguments. The first is the
+        Drag which was undergoing the snap animation. The second and third are
+        the x and y coordinates where the Drag was set to snap to. The fourth
+        is a boolean which is True if the snap animation was successfully
+        completed, and False if it was interrupted (e.g. from being grabbed in
+        the middle of snapping). For example, the following function sets
+        the drag's start_x and start_y position to its intended end position
+        if the snap animation was interrupted::
+
+            def snapped_callback(drag, x, y, completed):
+                if not completed:
+                    drag.start_x = x
+                    drag.start_y = y
+
+
     Except for `d`, all of the parameters are available as fields (with
     the same name) on the Drag object. In addition, after the drag has
     been rendered, the following fields become available:
@@ -265,6 +294,17 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
     `start_x`, `start_y`
         The drag start position of the Drag relative to its parent, in pixels.
 
+    `grab_x`, `grab_y`
+        The x and y positions, relative to its parent, where the drag was
+        picked up, in pixels.
+
+    `last_drop`
+        The last Drag that the current Drag can be dropped on if released,
+        or None if no valid Drag currently exists.
+
+    `snapping`
+        True if this Drag is in the middle of a snapping animation.
+
     `w`, `h`
         The width and height of the Drag's child, in pixels.
     """
@@ -273,13 +313,14 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
 
     focusable = True
 
-    drag_group = None
     old_position = None
     drag_offscreen = False
     activated = None
     alternate = None
     dragging = None
 
+    drag_group_weakref = None
+
     # The time a click started, or None if a click is not in progress.
     click_time = None
 
@@ -380,9 +421,20 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
         # The duration of a new snap animation to execute starting at
         # the next render() call
         self.target_at_delay = 0
+        # The warper used for the snap animation
+        self.snap_warper = None
+        # The time at which the current snap animation started
+        self.snap_start = 0
+        # The starting x and y coordinates of the current snap animation
+        self.snap_start_x = 0
+        self.snap_start_y = 0
+        # True if the drag is in the middle of a snapping animation
+        self.snapping = False
+        # A callback which is run when the drag completes a snap animation
+        self.snapped = properties.get("snapped", None)
 
         # The displayable we were last dropping on.
-        self.last_drop = None # type: renpy.display.core.Displayable|None
+        self.last_drop = None # type: renpy.display.displayable.Displayable|None
 
         # Did we move over the course of this drag?
         self.drag_moved = False
@@ -398,6 +450,11 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
             self.target_y = replaces.target_y
             self.target_at = replaces.target_at
             self.target_at_delay = replaces.target_at_delay
+            self.snap_warper = replaces.snap_warper
+            self.snap_start = replaces.snap_start
+            self.snap_start_x = replaces.snap_start_x
+            self.snap_start_y = replaces.snap_start_y
+            self.snapping = replaces.snapping
             self.grab_x = replaces.grab_x
             self.grab_y = replaces.grab_y
             self.last_x = replaces.last_x
@@ -416,27 +473,45 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
     def _draggable(self):
         return self.draggable
 
-    def snap(self, x, y, delay=0):
+    @property
+    def drag_group(self):
+        if self.drag_group_weakref is not None:
+            return self.drag_group_weakref()
+        else:
+            return None
+
+    @drag_group.setter
+    def drag_group(self, value):
+        if value is None:
+            self.drag_group_weakref = None
+        else:
+            self.drag_group_weakref = weakref.ref(value)
+
+    def snap(self, x, y, delay=0, warper=None):
         """
         :doc: drag_drop method
 
         Changes the position of the drag. If the drag is not showing,
         then the position change is instantaneous. Otherwise, the
-        position change takes `delay` seconds, and is animated as a
-        linear move.
+        position change takes `delay` seconds and an optional warper. If no
+        warper is provided, the transition is linear.
         """
 
-        if (type(x) is float) and self.parent_width is not None:
-            x = int(x * self.parent_width)
+        if self.parent_width is not None:
+            x = int(absolute.compute_raw(x, self.parent_width))
 
-        if (type(y) is float) and self.parent_height is not None:
-            y = int(y * self.parent_height)
+        if self.parent_height is not None:
+            y = int(absolute.compute_raw(y, self.parent_height))
 
         self.target_x = x
         self.target_y = y
 
         if self.x is not None:
             self.target_at_delay = delay
+            self.snap_warper = warper
+            self.snap_start_x = self.x
+            self.snap_start_y = self.y
+            self.snapping = True
         else:
             self.target_at = self.at
             self.x = x
@@ -505,7 +580,7 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
         self.set_style_prefix("idle_", True)
 
         # Set the style for joined_set
-        for i in [i[0] for i in self.drag_joined(self)]:
+        for i in [self.get_drag_from_name(i[0]) for i in self.drag_joined(self)]:
             i.set_style_prefix("selected_hover_", True)
 
         if self.last_drop is not None:
@@ -597,33 +672,36 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
             # Snap starts now
             self.target_at = at + self.target_at_delay
             self.target_at_delay = 0
+            # Record when the snap started
+            self.snap_start = at
             redraw(self, 0)
+            self.snapping = True
         elif self.target_at <= at or self.target_at <= self.at:
             # Snap complete
             self.x = self.target_x
             self.y = self.target_y
-        else:
+            self.snap_warper = None
+            self.snap_start = 0
+            if self.snapping:
+                run(self.snapped, self, self.target_x, self.target_y, True)
+            self.snapping = False
+        elif self.snapping:
             # Snap in progress
-            done = (at - self.at) / (self.target_at - self.at)
-            self.x = absolute(self.x + done * (self.target_x - self.x)) # type: ignore
-            self.y = absolute(self.y + done * (self.target_y - self.y)) # type: ignore
+            done = (at - self.snap_start) / (self.target_at - self.snap_start)
+            if self.snap_warper is not None:
+                done = self.snap_warper(done)
+            self.x = absolute((self.target_x-self.snap_start_x)*done + self.snap_start_x) # type: ignore
+            self.y = absolute((self.target_y-self.snap_start_y)*done + self.snap_start_y) # type: ignore
+
             redraw(self, 0)
 
         if self.draggable or self.clicked is not None:
 
             fx, fy, fw, fh = self.drag_handle
-
-            if isinstance(fx, float):
-                fx = int(fx * cw)
-
-            if isinstance(fy, float):
-                fy = int(fy * ch)
-
-            if isinstance(fw, float):
-                fw = int(fw * cw)
-
-            if isinstance(fh, float):
-                fh = int(fh * ch)
+            fx = int(absolute.compute_raw(fx, cw))
+            fy = int(absolute.compute_raw(fy, ch))
+            fw = int(absolute.compute_raw(fw, cw))
+            fh = int(absolute.compute_raw(fh, ch))
 
             mask = self.style.focus_mask
 
@@ -653,16 +731,34 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
 
         return rv
 
+    def get_drag_from_name(self, name):
+        if isinstance(name, Drag):
+            return name
+        else:
+            return self.drag_group.get_child_by_name(name)
+
     def event(self, ev, x, y, st):
 
+        grabbed = (renpy.display.focus.get_grab() is self)
+
+        if not grabbed:
+            rv = self.child.event(ev, x, y, st)
+            if rv is not None:
+                return rv
+
         if not self.is_focused():
-            return self.child.event(ev, x, y, st)
+            return None
 
         # Mouse, in parent-relative coordinates.
         par_x = int(self.last_x + x)
         par_y = int(self.last_y + y)
 
-        grabbed = (renpy.display.focus.get_grab() is self)
+        if grabbed and self.snapping:
+            # Stop the snap, since it was picked up
+            self.snapping = False
+            run(self.snapped, self, self.target_x, self.target_y, False)
+            self.snap_warper = None
+            self.snap_start = 0
 
         if (self.alternate is not None) and renpy.display.touch and map_event(ev, "drag_activate"):
             self.click_time = st
@@ -672,12 +768,12 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
 
         if grabbed:
             joined_offsets = self.drag_joined(self)
-            joined = [ i[0] for i in joined_offsets ] # type: list[Drag]
+            joined = [ self.get_drag_from_name(i[0]) for i in joined_offsets ] # type: list[Drag]
 
         elif self.draggable and map_event(ev, "drag_activate"):
 
             joined_offsets = self.drag_joined(self)
-            joined = [ i[0] for i in joined_offsets ] # type: list[Drag]
+            joined = [ self.get_drag_from_name(i[0]) for i in joined_offsets ] # type: list[Drag]
 
             if not joined:
                 raise renpy.display.core.IgnoreEvent()
@@ -692,14 +788,14 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
             # If we're not the only thing we're joined with, we
             # might need to adjust our grab point.
             for i, xo, yo in joined_offsets:
-                if i is self:
+                if self.get_drag_from_name(i) is self:
                     self.grab_x += xo
                     self.grab_y += yo
                     break
 
             self.drag_moved = False
-            self.start_x = par_x
-            self.start_y = par_y
+            self.start_x = par_x - self.grab_x
+            self.start_y = par_y - self.grab_y
 
             grabbed = True
 
@@ -732,8 +828,10 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
             if self.clicked is not None and map_event(ev, "drag_deactivate"):
 
                 self.click_time = None
-
-                rv = run(self.clicked)
+                try:
+                    rv = run(self.clicked, self)
+                except TypeError:
+                    rv = run(self.clicked)
                 if rv is not None:
                     return rv
 
@@ -741,12 +839,14 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
 
             return self.child.event(ev, x, y, st)
 
+
         # Handle moves by moving things relative to the grab point.
         if ev.type in (pygame.MOUSEMOTION, pygame.MOUSEBUTTONUP, pygame.MOUSEBUTTONDOWN):
 
             handled = True
 
-            if (not self.drag_moved) and (self.start_x != par_x or self.start_y != par_y):
+            if (not self.drag_moved) and (self.start_x != (par_x - self.grab_x)
+                    or self.start_y != (par_y - self.grab_y)):
                 self.drag_moved = True
                 self.click_time = None
 
@@ -756,6 +856,7 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
 
             if self.drag_moved:
                 for i, xo, yo in joined_offsets: # type: ignore
+                    i = self.get_drag_from_name(i)
 
                     new_x = int(par_x - self.grab_x + xo) # type: ignore
                     new_y = int(par_y - self.grab_y + yo) # type: ignore
@@ -825,6 +926,7 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
 
             self.last_drop = drop # type: ignore
 
+
         if self.drag_moved:
             self.update_style_prefix()
 
@@ -842,8 +944,6 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
 
             self.set_style_prefix("hover_", True)
 
-            self.grab_x = None
-            self.grab_y = None
             self.last_drop = None
 
             if self.drag_moved:
@@ -853,22 +953,35 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
                 if drag.dragged is not None:
                     rv = run(drag.dragged, joined, drop)
                     if rv is not None:
+                        self.grab_x = None
+                        self.grab_y = None
                         return rv
 
                 # Call the drop callback.
                 if drop is not None and drop.dropped is not None: # type: ignore
                     rv = run(drop.dropped, drop, joined) # type: ignore
                     if rv is not None:
+                        self.grab_x = None
+                        self.grab_y = None
                         return rv
 
             else:
 
                 # Call the clicked callback.
                 if self.clicked:
-                    rv = run(self.clicked)
+                    try:
+                        rv = self.clicked(self)
+                    except TypeError:
+                        rv = run(self.clicked)
                     if rv is not None:
+                        self.grab_x = None
+                        self.grab_y = None
                         return rv
 
+            self.grab_x = None
+            self.grab_y = None
+            self.drag_moved = False
+
         if handled:
             raise renpy.display.core.IgnoreEvent()
 
@@ -938,7 +1051,8 @@ class DragGroup(renpy.display.layout.MultiBox):
         """
         :doc: drag_drop method
 
-        Adds `child`, which must be a Drag, to this DragGroup.
+        Adds `child`, which must be a Drag, to this DragGroup. This child
+        will be added above all other children of this DragGroup.
         """
 
         if not isinstance(child, Drag):
@@ -949,6 +1063,11 @@ class DragGroup(renpy.display.layout.MultiBox):
         self.sorted = False
         renpy.display.render.invalidate(self)
 
+        child.drag_group = self
+
+        if renpy.config.drag_group_add_top:
+            child.top()
+
     def remove(self, child):
         """
         :doc: drag_drop method
