@@ -135,6 +135,31 @@ class Focus(object):
 
         return False
 
+    def inset_rect(self):
+        """
+        Returns the rectangle with the keyboard focus insets applied.
+        """
+
+        x = self.x
+        y = self.y
+        w = self.w
+        h = self.h
+
+        insets = self.widget.style.keyboard_focus_insets
+
+        if insets is not None:
+            x += insets[0]
+            y += insets[1]
+            w -= insets[0] + insets[2]
+            h -= insets[1] + insets[3]
+
+        if w < 1:
+            w = 1
+        if h < 1:
+            h = 1
+
+        return x, y, w, h
+
 
 # The current focus argument.
 argument = None
@@ -409,7 +434,7 @@ def before_interact(roots):
         max_default_focus_name = None
 
     # Should we do the max_default logic?
-    should_max_default = (renpy.display.interface.last_event is None) or (renpy.display.interface.last_event.type not in [ pygame.MOUSEBUTTONDOWN, pygame.MOUSEBUTTONUP, pygame.MOUSEMOTION ])
+    should_max_default = renpy.display.interface.input_event_time > renpy.display.interface.mouse_event_time + .1
 
     # Is this an explicit change, using the override operation?
     explicit = False
@@ -442,7 +467,9 @@ def before_interact(roots):
         current = max_default_focus
         set_grab(None)
         set_focused(max_default_focus, None, max_default_screen)
-        old_max_default = max_default
+        explicit = True
+
+    old_max_default = max_default
 
     # Try to find the current focus.
     if current is not None:
@@ -641,6 +668,18 @@ def focus_extreme(xmul, ymul, wmul, hmul):
         return change_focus(max_focus)
 
 
+def check_keyboard_focus():
+    """
+    If the current widget is not keyboard focusable, clears
+    the focus.
+    """
+
+    current = get_focused()
+
+    if current and not current.style.keyboard_focus:
+        change_focus(None)
+
+
 # This calculates the distance between two points, applying
 # the given fudge factors. The distance is left squared.
 def points_dist(x0, y0, x1, y1, xfudge, yfudge):
@@ -714,6 +753,8 @@ def focus_nearest(from_x0, from_y0, from_x1, from_y1,
     if not focus_list:
         return
 
+    check_keyboard_focus()
+
     # No widget focused.
     current = get_focused()
 
@@ -753,10 +794,12 @@ def focus_nearest(from_x0, from_y0, from_x1, from_y1,
         focus_extreme(xmul, ymul, wmul, hmul)
         return
 
-    fx0 = from_focus.x + from_focus.w * from_x0
-    fy0 = from_focus.y + from_focus.h * from_y0
-    fx1 = from_focus.x + from_focus.w * from_x1
-    fy1 = from_focus.y + from_focus.h * from_y1
+    from_focus_x, from_focus_y, from_focus_w, from_focus_h = from_rect = from_focus.inset_rect()
+
+    fx0 = from_focus_x + from_focus_w * from_x0
+    fy0 = from_focus_y + from_focus_h * from_y0
+    fx1 = from_focus_x + from_focus_w * from_x1
+    fy1 = from_focus_y + from_focus_h * from_y1
 
     placeless = None
     new_focus = None
@@ -779,13 +822,15 @@ def focus_nearest(from_x0, from_y0, from_x1, from_y1,
         if f.x is False:
             continue
 
-        if not condition(from_focus, f):
+        f_x, f_y, f_w, f_h = to_rect = f.inset_rect()
+
+        if not condition(from_rect, to_rect):
             continue
 
-        tx0 = f.x + f.w * to_x0
-        ty0 = f.y + f.h * to_y0
-        tx1 = f.x + f.w * to_x1
-        ty1 = f.y + f.h * to_y1
+        tx0 = f_x + f_w * to_x0
+        ty0 = f_y + f_h * to_y0
+        tx1 = f_x + f_w * to_x1
+        ty1 = f_y + f_h * to_y1
 
         dist = line_dist(fx0, fy0, fx1, fy1,
                          tx0, ty0, tx1, ty1)
@@ -814,6 +859,8 @@ def focus_ordered(delta):
     candidates = [ ]
     index = 0
 
+    check_keyboard_focus()
+
     current = get_focused()
     current_index = None
 
@@ -870,30 +917,31 @@ def key_handler(ev):
 
     else:
 
+
         if map_event(ev, 'focus_right'):
             return focus_nearest(0.9, 0.1, 0.9, 0.9,
                                  0.1, 0.1, 0.1, 0.9,
                                  verti_line_dist,
-                                 lambda old, new : old.x + old.w <= new.x,
+                                 lambda old, new : old[0] + old[2] <= new[0],
                                  -1, 0, 0, 0)
 
         if map_event(ev, 'focus_left'):
             return focus_nearest(0.1, 0.1, 0.1, 0.9,
                                  0.9, 0.1, 0.9, 0.9,
                                  verti_line_dist,
-                                 lambda old, new : new.x + new.w <= old.x,
+                                 lambda old, new : new[0] + new[2] <= old[0],
                                  1, 0, 1, 0)
 
         if map_event(ev, 'focus_up'):
             return focus_nearest(0.1, 0.1, 0.9, 0.1,
                                  0.1, 0.9, 0.9, 0.9,
                                  horiz_line_dist,
-                                 lambda old, new : new.y + new.h <= old.y,
+                                 lambda old, new : new[1] + new[3] <= old[1],
                                  0, 1, 0, 1)
 
         if map_event(ev, 'focus_down'):
             return focus_nearest(0.1, 0.9, 0.9, 0.9,
                                  0.1, 0.1, 0.9, 0.1,
                                  horiz_line_dist,
-                                 lambda old, new : old.y + old.h <= new.y,
+                                 lambda old, new : old[1] + old[3] <= new[1],
                                  0, -1, 0, 0)
