@@ -34,6 +34,7 @@ import threading
 import time
 import io
 import os.path
+import sys
 
 import pygame_sdl2
 import renpy
@@ -145,8 +146,7 @@ class Cache(object):
         cache, in pixels.
         """
 
-        with self.lock:
-            return self.cache_size
+        return self.cache_size
 
     def get_current_size(self, generations):
         """
@@ -190,18 +190,16 @@ class Cache(object):
     # Clears out the cache.
     def clear(self):
 
-        self.lock.acquire()
+        with self.lock:
 
-        self.preloads = [ ]
+            self.preloads = [ ]
 
-        self.cache = { }
-        self.cache_size = 0
+            self.cache = { }
+            self.cache_size = 0
 
-        self.first_preload_in_tick = True
+            self.first_preload_in_tick = True
 
-        self.added.clear()
-
-        self.lock.release()
+            self.added.clear()
 
     def get_renders(self):
         """
@@ -210,14 +208,8 @@ class Cache(object):
 
         Render = renpy.display.render.Render
 
-        rv = [ ]
-
         with self.lock:
-            for ce in self.cache.values():
-                if isinstance(ce.texture, Render):
-                    rv.append(ce.texture)
-
-        return rv
+            return [ ce.texture for ce in self.cache.values() if isinstance(ce.texture, Render) ]
 
     # Increments time, and clears the list of images to be
     # preloaded.
@@ -266,6 +258,9 @@ class Cache(object):
             if image.pixel_perfect:
                 rv.add_property("pixel_perfect", True)
 
+            if ce.bounds == (0, 0, ce.width, ce.height):
+                rv.cached_texture = ce.texture
+
             return rv
 
         if render:
@@ -324,21 +319,23 @@ class Cache(object):
             else:
                 bounds = (0, 0, w, h)
 
+            ce = CacheEntry(image, surf, bounds)
+
             with self.lock:
 
-                ce = CacheEntry(image, surf, bounds)
-
-                if image in self.cache:
-                    self.kill(self.cache[image])
+                old_ce = self.cache.get(image, None)
 
                 self.cache[image] = ce
                 self.cache_size += ce.size()
 
-                if renpy.config.debug_image_cache:
-                    if predict:
-                        renpy.display.ic_log.write("Added %r (%.02f%%)", ce.what, 100.0 * self.get_total_size() / self.cache_limit)
-                    else:
-                        renpy.display.ic_log.write("Total Miss %r", ce.what)
+                if old_ce is not None:
+                    self.cache_size -= old_ce.size()
+
+            if renpy.config.debug_image_cache:
+                if predict:
+                    renpy.display.ic_log.write("Added %r (%.02f%%)", ce.what, 100.0 * self.get_total_size() / self.cache_limit)
+                else:
+                    renpy.display.ic_log.write("Total Miss %r", ce.what)
 
             renpy.display.render.mutated_surface(ce.surf)
 
@@ -384,8 +381,7 @@ class Cache(object):
             return make_render(ce)
 
         if (ce.surf is None) and (ce.texture is None):
-            with self.lock:
-                self.kill(ce)
+            self.kill(ce)
 
         return rv
 
@@ -396,8 +392,10 @@ class Cache(object):
         if ce.surf is not None:
             renpy.display.draw.mutated_surface(ce.surf)
 
-        self.cache_size -= ce.size()
-        del self.cache[ce.what]
+        with self.lock:
+            if self.cache.get(ce.what, None) is ce:
+                del self.cache[ce.what]
+                self.cache_size -= ce.size()
 
         if renpy.config.debug_image_cache:
             renpy.display.ic_log.write("Removed %r", ce.what)
@@ -416,7 +414,10 @@ class Cache(object):
         # If we're outside the cache limit, we need to go and start
         # killing off some of the entries until we're back inside it.
 
-        for ce in sorted(self.cache.values(), key=lambda a : a.time):
+        with self.lock:
+            cache_values = list(self.cache.values())
+
+        for ce in sorted(cache_values, key=lambda a : a.time):
 
             if ce.time == self.time:
                 # If we're bigger than the limit, and there's nothing
@@ -471,21 +472,19 @@ class Cache(object):
         if not isinstance(im, ImageBase):
             return
 
-        with self.lock:
+        if im in self.added:
+            return
 
-            if im in self.added:
-                return
+        self.added.add(im)
 
-            self.added.add(im)
+        ce = self.cache.get(im, None)
 
-            ce = self.cache.get(im, None)
-
-            if ce and ce.texture:
-                ce.time = self.time
-                in_cache = True
-            else:
-                self.preloads.append(im)
-                in_cache = False
+        if ce and ce.texture:
+            ce.time = self.time
+            in_cache = True
+        else:
+            self.preloads.append(im)
+            in_cache = False
 
         if not in_cache:
 
@@ -520,18 +519,17 @@ class Cache(object):
 
             # If the size of the current generation is bigger than the
             # total cache size, stop preloading.
-            with self.lock:
 
-                # If the cache is overfull, clean it out.
-                if not self.cleanout():
+            # If the cache is overfull, clean it out.
+            if not self.cleanout():
 
-                    if renpy.config.debug_image_cache:
-                        for i in self.preloads:
-                            renpy.display.ic_log.write("Overfull %r", i)
+                if renpy.config.debug_image_cache:
+                    for i in self.preloads:
+                        renpy.display.ic_log.write("Overfull %r", i)
 
-                    self.preloads = [ ]
+                self.preloads = [ ]
 
-                    break
+                return
 
             try:
                 image = self.preloads.pop(0)
@@ -544,8 +542,7 @@ class Cache(object):
             except Exception:
                 pass
 
-        with self.lock:
-            self.cleanout()
+        self.cleanout()
 
     def add_load_log(self, filename):
 
@@ -583,6 +580,10 @@ class ImageBase(renpy.display.displayable.Displayable):
     optimize_bounds = False
     oversample = 1
     pixel_perfect = False
+    obsolete = True
+
+    obsolete_list = [ ]
+
 
     # If the image failed to load, a placeholder used to report the error.
     fail = None
@@ -606,6 +607,14 @@ class ImageBase(renpy.display.displayable.Displayable):
         super(ImageBase, self).__init__(**properties)
         self.identity = (type(self).__name__,) + args
 
+        if self.obsolete and renpy.game.context().init_phase:
+            frame = sys._getframe(2)
+            filename = frame.f_code.co_filename
+            line = frame.f_lineno
+            classname = type(self).__name__
+
+            self.obsolete_list.append((filename, line, classname))
+
     def __hash__(self):
         return hash(self.identity)
 
@@ -677,6 +686,8 @@ class Image(ImageBase):
     This image manipulator loads an image from a file.
     """
 
+    obsolete = False
+
     is_svg = False
     dpi = 96
 
@@ -687,7 +698,7 @@ class Image(ImageBase):
 
         if "@" in filename:
             base = filename.rpartition(".")[0]
-            extras = base.partition("@")[2].split(",")
+            extras = base.rpartition("@")[2].partition("/")[0].split(",")
 
             for i in extras:
                 try:
@@ -807,6 +818,8 @@ class Data(ImageBase):
         loaded from disk.)
     """
 
+    obsolete = False
+
     def __init__(self, data, filename, **properties):
         super(Data, self).__init__(data, filename, **properties)
         self.data = data
@@ -822,6 +835,8 @@ class Data(ImageBase):
 
 class ZipFileImage(ImageBase):
 
+    obsolete = False
+
     def __init__(self, zipfilename, filename, mtime=0, **properties):
         super(ZipFileImage, self).__init__(zipfilename, filename, mtime, **properties)
 
