@@ -693,9 +693,8 @@ class DynamicImage(renpy.display.displayable.Displayable):
 
         if raw_target._duplicatable:
             target = raw_target._duplicate(self._args)
-
             if not self._duplicatable:
-                self.target._unique()
+                target._unique()
 
         self.raw_target = raw_target
         self.target = target
@@ -732,7 +731,8 @@ class DynamicImage(renpy.display.displayable.Displayable):
     def _unique(self):
         if self.target is not None:
             self.target._unique()
-            self._duplicatable = False
+
+        self._duplicatable = False
 
     def _in_current_store(self):
         rv = self._copy()
