@@ -1,4 +1,4 @@
-# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -36,7 +36,7 @@ from renpy.display.render import render
 cached = set()
 
 
-class ImageMapCrop(renpy.display.core.Displayable):
+class ImageMapCrop(renpy.display.displayable.Displayable):
     """
     This handles the cropping of uncached imagemap components.
     """
@@ -51,11 +51,11 @@ class ImageMapCrop(renpy.display.core.Displayable):
         return [ self.child ]
 
     def render(self, width, height, st, at):
-        cr = render(self.child, width, height, st, at)
+        cr = render(self.child, renpy.config.screen_width, renpy.config.screen_height, st, at)
         return cr.subsurface(self.rect)
 
 
-class ImageCacheCrop(renpy.display.core.Displayable):
+class ImageCacheCrop(renpy.display.displayable.Displayable):
     """
     This handles the cropping of an imagemap component.
     """
