@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,9 +21,12 @@
 
 # This file contains the routines that manage image prediction.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-import renpy.display
+
+
+import renpy
 
 # Called to indicate an image should be loaded or preloaded. This is
 # a function that takes an image manipulator, set by reset and predict,
@@ -100,7 +103,11 @@ def prediction_coroutine(root_widget):
 
     # Predict displayables given to renpy.start_predict.
     for d in renpy.store._predict_set:
-        displayable(d)
+        try:
+            displayable(d)
+        except Exception:
+            if renpy.config.debug_prediction:
+                raise
 
         predicting = False
         yield True
@@ -122,11 +129,11 @@ def prediction_coroutine(root_widget):
     if len(renpy.game.contexts) >= 2:
         sls = renpy.game.contexts[-2].scene_lists
 
-        for l in sls.layers.itervalues():
+        for l in sls.layers.values():
             for sle in l:
                 try:
                     displayable(sle.displayable)
-                except:
+                except Exception:
                     pass
 
     else:
@@ -138,10 +145,26 @@ def prediction_coroutine(root_widget):
     while not (yield False):
         continue
 
+    predicting = True
+
+    for i in renpy.config.expensive_predict_callbacks:
+        done = False
+        while not done:
+            if not i():
+                done = True
+
+            predicting = False
+            yield False
+            predicting = True
+
+    predicted_screens = [ ]
+
     # Predict screens given with renpy.start_predict_screen.
-    for name, value in renpy.store._predict_screen.items():
+    for name, value in list(renpy.store._predict_screen.items()):
         args, kwargs = value
 
+        predicted_screens.append((name, args, kwargs))
+
         renpy.display.screen.predict_screen(name, *args, **kwargs)
 
         predicting = False
@@ -154,13 +177,16 @@ def prediction_coroutine(root_widget):
 
     try:
         root_widget.visit_all(lambda i : i.predict_one_action())
-    except:
-        pass
+    except Exception:
+        if renpy.config.debug_prediction:
+            import traceback
+
+            print("While predicting actions.")
+            traceback.print_exc()
+            print()
 
     predicting = False
 
-    predicted_screens = [ ]
-
     # Predict the screens themselves.
     for t in screens:
 
@@ -179,12 +205,7 @@ def prediction_coroutine(root_widget):
 
         predicting = True
 
-        try:
-            renpy.display.screen.predict_screen(name, *args, **kwargs)
-        except:
-            if renpy.config.debug_image_cache:
-                renpy.display.ic_log.write("While predicting screen %s %r", name, kwargs)
-                renpy.display.ic_log.exception()
+        renpy.display.screen.predict_screen(name, *args, **kwargs)
 
         predicting = False
 
