@@ -1,4 +1,4 @@
-# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -33,14 +33,14 @@ import renpy
 from renpy.display.render import render
 
 
-class Transition(renpy.display.core.Displayable):
+class Transition(renpy.display.displayable.Displayable):
     """
     This is the base class of most transitions. It takes care of event
     dispatching.
     """
 
-    new_widget = None # type:renpy.display.core.Displayable|None
-    old_widget = None # type:renpy.display.core.Displayable|None
+    new_widget = None # type:renpy.display.displayable.Displayable|None
+    old_widget = None # type:renpy.display.displayable.Displayable|None
 
     def __init__(self, delay, **properties):
         super(Transition, self).__init__(**properties)
@@ -57,6 +57,12 @@ class Transition(renpy.display.core.Displayable):
     def visit(self):
         return [ self.new_widget, self.old_widget ] # E1101
 
+    def get_placement(self):
+        if renpy.config.transitions_use_child_placement:
+            return self.new_widget.get_placement()
+        else:
+            return super(Transition, self).get_placement()
+
 
 def null_render(d, width, height, st, at):
 
@@ -158,7 +164,7 @@ class MultipleTransition(Transition):
         self.events = False
 
     def visit(self):
-        return [ i for i in self.screens if isinstance(i, renpy.display.core.Displayable)] + self.transitions
+        return [ i for i in self.screens if isinstance(i, renpy.display.displayable.Displayable)] + self.transitions
 
     def event(self, ev, x, y, st):
 
@@ -331,9 +337,9 @@ class Dissolve(Transition):
         The time the dissolve will take.
 
     `time_warp`
-        A function that adjusts the timeline. If not None, this should be a
-        function that takes a fractional time between 0.0 and 1.0, and returns
-        a number in the same range.
+        A :ref:`function that adjusts the timeline <warpers>`. If not None, this
+        should be a function that takes a fractional time between 0.0 and 1.0,
+        and returns a number in the same range.
 
     `mipmap`
         When the dissolve will be scaled to less than half its natural size,
@@ -377,8 +383,12 @@ class Dissolve(Transition):
         bottom = render(self.old_widget, width, height, st, at)
         top = render(self.new_widget, width, height, st, at)
 
-        width = max(top.width, bottom.width)
-        height = max(top.height, bottom.height)
+        if renpy.config.dissolve_shrink:
+            width = min(top.width, bottom.width)
+            height = min(top.height, bottom.height)
+        else:
+            width = max(top.width, bottom.width)
+            height = max(top.height, bottom.height)
 
         rv = renpy.display.render.Render(width, height)
 
@@ -436,9 +446,9 @@ class ImageDissolve(Transition):
         If True, black pixels will dissolve in before white pixels.
 
     `time_warp`
-        A function that adjusts the timeline. If not None, this should be a
-        function that takes a fractional time between 0.0 and 1.0, and returns
-        a number in the same range.
+        A :ref:`function that adjusts the timeline <warpers>`. If not None, this
+        should be a function that takes a fractional time between 0.0 and 1.0,
+        and returns a number in the same range.
 
     `mipmap`
         When the dissolve will be scaled to less than half its natural size,
@@ -542,8 +552,12 @@ class ImageDissolve(Transition):
         bottom = render(self.old_widget, width, height, st, at)
         top = render(self.new_widget, width, height, st, at)
 
-        width = min(bottom.width, top.width, image.width)
-        height = min(bottom.height, top.height, image.height)
+        if renpy.config.dissolve_shrink:
+            width = min(bottom.width, top.width, image.width)
+            height = min(bottom.height, top.height, image.height)
+        else:
+            width = max(bottom.width, top.width, image.width)
+            height = max(bottom.height, top.height, image.height)
 
         rv = renpy.display.render.Render(width, height)
 
@@ -666,8 +680,12 @@ class AlphaDissolve(Transition):
         bottom = render(self.old_widget, width, height, st, at)
         top = render(self.new_widget, width, height, st, at)
 
-        width = min(bottom.width, top.width)
-        height = min(bottom.height, top.height)
+        if renpy.config.dissolve_shrink:
+            width = min(bottom.width, top.width, image.width)
+            height = min(bottom.height, top.height, image.height)
+        else:
+            width = max(bottom.width, top.width)
+            height = max(bottom.height, top.height)
 
         control = render(self.control, width, height, st, at)
 
