@@ -288,10 +288,10 @@ def render_movie(channel, width, height):
 
 def default_play_callback(old, new): # @UnusedVariable
 
-    renpy.audio.music.play(new._play, channel=new.channel, loop=new.loop, synchro_start=True)
+    renpy.audio.music.play(new._play, channel=new.channel, loop=new.loop)
 
     if new.mask:
-        renpy.audio.music.play(new.mask, channel=new.mask_channel, loop=new.loop, synchro_start=True)
+        renpy.audio.music.play(new.mask, channel=new.mask_channel, loop=new.loop)
 
 # A serial number that's used to generated movie channels.
 movie_channel_serial = 0
@@ -517,6 +517,27 @@ class Movie(renpy.display.displayable.Displayable):
 
         self.group = group
 
+        if self.image and self.image._duplicatable:
+            self._duplicatable = True
+
+        if self.start_image and self.start_image._duplicatable:
+            self._duplicatable = True
+
+
+    def _duplicate(self, args):
+        if not self._duplicatable:
+            return self
+
+        rv = self._copy(args)
+
+        if rv.image and rv.image._duplicatable:
+            rv.image = rv.image._duplicate(args)
+
+        if rv.start_image and rv.start_image._duplicatable:
+            rv.start_image = rv.start_image._duplicate(args)
+
+        return rv
+
     def _handles_event(self, event):
         return event == "show"
 
@@ -592,6 +613,9 @@ class Movie(renpy.display.displayable.Displayable):
         else:
             old_play = old._play
 
+        if (self._play is None) and (old_play is None):
+            return
+
         if (self._play != old_play) or renpy.config.replay_movie_sprites:
             if self._play:
 
@@ -641,29 +665,47 @@ def playing():
     return
 
 
+# A map from a channel to the movie playing on it in the last
+# interaction. Used to restart looping movies.
+last_channel_movie = { }
+
 def update_playing():
     """
     Calls play/stop on Movie displayables.
     """
 
+    global last_channel_movie
+
     old_channel_movie = renpy.game.context().movie
 
     for c, m in channel_movie.items():
 
         old = old_channel_movie.get(c, None)
+        last = last_channel_movie.get(c, None)
 
         if (c in reset_channels) and renpy.config.replay_movie_sprites:
             m.play(old)
         elif old is not m:
             m.play(old)
+        elif m.loop and last is not m:
+            m.play(last)
+
+    stopped = set()
+
+    for c, m in last_channel_movie.items():
+        if c not in channel_movie:
+            stopped.add(c)
+            m.stop()
 
     for c, m in old_channel_movie.items():
         if c not in channel_movie:
-            m.stop()
+            if c not in stopped:
+                m.stop()
 
-    renpy.game.context().movie = dict(channel_movie)
+    renpy.game.context().movie = last_channel_movie = dict(channel_movie)
     reset_channels.clear()
 
+
 def frequent():
     """
     Called to update the video playback. Returns true if a video refresh is
