@@ -1,4 +1,4 @@
-# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -52,7 +52,8 @@ STYLE_PREFIXES = [
 parser = None
 
 # The names of all statements known to SL.
-statement_names = set()
+# a map of "parser_name": Parser
+statements = dict()
 
 # A list of statements that are valid anywhere a child can be placed.
 all_child_statements = [ ]
@@ -153,7 +154,7 @@ class Parser(object):
         self.keyword = { }
         self.children = { }
 
-        statement_names.add(name)
+        statements[name] = self
 
         # True if this parser takes "as".
         self.variable = False
@@ -277,17 +278,16 @@ class Parser(object):
             if name not in self.keyword:
                 if name == "continue" or name == "break":
                     l.error("The %s statement may only appear inside a for statement, or an if statement inside a for statement." % name)
-                elif name in statement_names:
+                elif name in statements:
                     l.error('The %s statement is not a valid child of the %s statement.' % (name, self.name))
                 else:
                     l.error('%r is not a keyword argument or valid child of the %s statement.' % (name, self.name))
 
-            if name == "at" and l.keyword("transform"):
+            if name == "at" and l.match_multiple("transform", ":"):
 
                 if target.atl_transform is not None:
                     l.error("More than one 'at transform' block is given.")
 
-                l.require(":")
                 l.expect_eol()
                 l.expect_block("ATL block")
                 expr = renpy.atl.parse_atl(l.subblock_lexer())
@@ -448,6 +448,29 @@ class Parser(object):
 
         return self
 
+    def copy_properties(self, name):
+        global parser
+        parser = self
+
+        parser_to_copy = statements.get(name, None)
+        if parser_to_copy is None:
+            raise Exception("{!r} is not a known screen statement".format(name))
+
+        for p in parser_to_copy.positional:
+            Positional(p.name)
+
+        for v in set(parser_to_copy.keyword.values()):
+            if isinstance(v, Keyword):
+                Keyword(v.name)
+
+            elif isinstance(v, Style):
+                Style(v.name)
+
+            elif isinstance(v, PrefixStyle):
+                PrefixStyle(v.prefix, v.name)
+
+        return self
+
 
 def add(thing):
     parser.add(thing)
@@ -554,6 +577,10 @@ def register_sl_displayable(*args, **kwargs):
 
         These correspond to groups of :doc:`style_properties`. Group can
         also be "ui", in which case it adds the :ref:`common ui properties <common-properties>`.
+
+    .. method:: copy_properties(name)
+
+        Adds all styles and positional/keyword arguments that can be passed to the `name` screen statement.
     """
 
     kwargs.setdefault("unique", False)
