@@ -37,6 +37,8 @@ import renpy.text.font as font
 import renpy.text.extras as extras
 from renpy.text.emoji_trie import emoji, UNQUALIFIED
 
+from renpy.gl2.gl2polygon import Polygon
+
 from _renpybidi import log2vis, WRTL, RTL, ON # @UnresolvedImport
 
 BASELINE = -65536
@@ -67,6 +69,72 @@ class Blit(object):
         return "<Blit ({0}, {1}, {2}, {3}) {4}>".format(self.x, self.y, self.w, self.h, self.alpha)
 
 
+class TextMeshDisplayable(renpy.display.core.Displayable):
+    """
+    This is a displayable that renders a mesh of text.
+    """
+
+    def __init__(self, width, height, tex, shader, mesh, uniforms):
+        super(TextMeshDisplayable, self).__init__()
+
+        self.width = width
+        self.height = height
+        self.tex = tex
+        self.shader = shader
+        self.mesh = mesh
+        self.uniforms = uniforms
+
+    def render(self, width, height, st, at):
+
+        rv = renpy.display.render.Render(self.width, self.height)
+
+        rv.add_shader("renpy.texture")
+
+        for i in self.shader:
+            rv.add_shader(i)
+
+        for k, v, is_displayable in self.uniforms:
+
+            if is_displayable:
+                v = renpy.display.render.render(v, self.width, self.height, st, at)
+                rv.depends_on(v)
+
+            rv.add_uniform(k, v)
+
+        if not isinstance(self.tex, renpy.display.render.Render):
+
+            # Single texture case.
+
+            rv.absolute_blit(self.tex, (0, 0))
+            rv.mesh = self.mesh
+
+        else:
+
+            for model, x, y, focus, main in self.tex.children:
+                tex = model.uniforms["tex0"]
+                w, h = tex.get_size()
+
+                # Adjust the size for the texture borders.
+                w -= tex.bl + tex.br
+                h -= tex.bt + tex.bb
+
+                cmesh = self.mesh.crop(Polygon.rectangle(x, y+h, x+w, y))
+
+                cmesh.remap_texture(
+                    0, 0, self.width, self.height,
+                    x - tex.bl, y - tex.bt, w + tex.bl + tex.br, h + tex.bt + tex.bb,
+                )
+
+                cr = renpy.display.render.Render(self.width, self.height)
+                cr.mesh = cmesh
+
+                cr.absolute_blit(tex, (0, 0))
+                rv.absolute_blit(cr, (0, 0))
+
+
+        return rv
+
+
 def outline_blits(blits, outline):
     """
     Given a list of blits, adjusts it for the given outline size. That means
@@ -203,6 +271,7 @@ class TextSegment(object):
             self.shaper = source.shaper
             self.instance = source.instance
             self.axis = source.axis
+            self.shader = source.shader
 
         else:
             self.hyperlink = 0
@@ -211,6 +280,7 @@ class TextSegment(object):
             self.ruby_bottom = False
             self.ignore = False
             self.default_font = True
+            self.shader = None
 
     def __repr__(self):
         return "<TextSegment font={font}, size={size}, bold={bold}, italic={italic}, underline={underline}, color={color}, black_color={black_color}, hyperlink={hyperlink}, vertical={vertical}>".format(**self.__dict__)
@@ -239,8 +309,8 @@ class TextSegment(object):
             self.underline = 0
 
         self.strikethrough = layout.scale_int(style.strikethrough)
-        self.color = style.color
-        self.black_color = style.black_color
+        self.color = style.color or self.color
+        self.black_color = style.black_color or self.black_color
         self.hyperlink = None
         self.kerning = layout.scale(style.kerning)
         self.outline_color = None
@@ -255,6 +325,8 @@ class TextSegment(object):
         self.axis = style.axis
         self.instance = style.instance
 
+        self.shader = renpy.text.shader.get_textshader(style.textshader)
+
     # From here down is the public glyph API.
 
     def glyphs(self, s, layout):
@@ -283,6 +355,10 @@ class TextSegment(object):
         elif self.ruby_top:
             textsupport.mark_ruby_top(rv)
 
+        if self.shader is not None:
+            for g in rv:
+                g.shader = self.shader
+
         return rv
 
     def draw(self, glyphs, di, xo, yo, layout):
@@ -371,27 +447,32 @@ class SpaceSegment(object):
     A segment that's used to render horizontal or vertical whitespace.
     """
 
-    def __init__(self, ts, width=0, height=0):
+    def __init__(self, ts, width=0.0, height=0.0):
         """
         `ts`
             The text segment that this SpaceSegment follows.
         """
 
-        self.glyph = glyph = textsupport.Glyph()
+        self.width = width
+        self.height = height
+        self.ts = ts
+
+
+    def glyphs(self, s, layout):
+        glyph = textsupport.Glyph()
 
         glyph.character = 0
         glyph.ascent = 1
-        glyph.line_spacing = height
-        glyph.advance = width
-        glyph.width = width
+        glyph.line_spacing = layout.scale_int(self.height)
+        glyph.advance = layout.scale_int(self.width)
+        glyph.width = layout.scale_int(self.width)
 
-        if ts.hyperlink:
-            glyph.hyperlink = ts.hyperlink
+        if self.ts.hyperlink:
+            glyph.hyperlink = self.ts.hyperlink
 
-        self.cps = ts.cps
+        glyph.shader = self.ts.shader
 
-    def glyphs(self, s, layout):
-        return [ self.glyph ]
+        return [ glyph ]
 
     def bounds(self, glyphs, bounds, layout):
         return bounds
@@ -402,10 +483,12 @@ class SpaceSegment(object):
         return
 
     def assign_times(self, gt, glyphs):
-        if self.cps != 0:
-            gt += 1.0 / self.cps
+        cps = self.ts.cps
 
-        self.glyph.time = gt
+        if cps != 0:
+            gt += 1.0 / cps
+
+        glyphs[0].time = gt
         return gt
 
 
@@ -425,13 +508,11 @@ class DisplayableSegment(object):
 
         self.width, self.height = rend.get_size()
 
-        if isinstance(d, renpy.display.behavior.CaretBlink):
-            self.width = 0
-
         self.hyperlink = ts.hyperlink
         self.cps = ts.cps
         self.ruby_top = ts.ruby_top
         self.ruby_bottom = ts.ruby_bottom
+        self.shader = ts.shader
 
     def glyphs(self, s, layout):
 
@@ -446,8 +527,9 @@ class DisplayableSegment(object):
         glyph.character = 0xfffc
         glyph.ascent = 0
         glyph.line_spacing = h
-        glyph.advance = w
+        glyph.advance = 0 if isinstance(self.d, renpy.display.behavior.CaretBlink) else w
         glyph.width = w
+        glyph.shader = self.shader
 
         if self.hyperlink:
             glyph.hyperlink = self.hyperlink
@@ -589,6 +671,11 @@ class Layout(object):
         self.width = width
         self.height = height
 
+        # The default characters-per-second for this displayable.
+        self.cps = style.slow_cps
+        if self.cps is None or self.cps is True:
+            self.cps = renpy.game.preferences.text_cps
+
         width = self.scale_int(width)
         height = self.scale_int(height)
 
@@ -623,6 +710,9 @@ class Layout(object):
         # The time at which the next glyph will be displayed.
         gt = 0.0
 
+        # The index of these glyphs.
+        index = 0.0
+
         # 2. Breaks the text into a list of paragraphs, where each paragraph is
         # represented as a list of (Segment, text string) tuples.
         #
@@ -672,6 +762,7 @@ class Layout(object):
             # A list of (segment, list of glyph) pairs.
             seg_glyphs = [ ]
 
+
             for ts, s in p:
                 glyphs = ts.glyphs(s, self)
 
@@ -685,6 +776,7 @@ class Layout(object):
             # linebreaking algorithms.
             if rtl:
                 par_glyphs.reverse()
+                seg_glyphs.reverse()
                 for ts, glyphs in seg_glyphs:
                     glyphs.reverse()
 
@@ -735,6 +827,7 @@ class Layout(object):
                     if self.start_segment is ts:
                         started = True
                     else:
+                        textsupport.assign_times(-3600, 0.0, glyphs)
                         continue
 
                 if ts is self.end_segment:
@@ -745,6 +838,8 @@ class Layout(object):
                 else:
                     gt = ts.assign_times(gt, glyphs)
 
+            index = textsupport.assign_index(index, par_glyphs)
+
             # RTL - Reverse the glyphs in each line, back to RTL order,
             # now that we have lines.
             if rtl:
@@ -822,6 +917,12 @@ class Layout(object):
         if self.has_ruby:
             textsupport.place_ruby(all_glyphs, self.scale_int(style.ruby_style.yoffset), self.scale_int(style.altruby_style.yoffset), sw, sh)
 
+        # Determine the set of textshaders.
+        if renpy.display.draw.info.get("models", False):
+            self.textshaders = textsupport.get_textshader_set(all_glyphs)
+        else:
+            self.textshaders = None
+
         # Check for glyphs that are being drawn out of bounds, because the font
         # or anti-aliasing or whatever makes them bigger than the bounding box. If
         # we have them, grow the bounding box.
@@ -841,9 +942,17 @@ class Layout(object):
         # A map from (outline, color) to a texture.
         self.textures = { }
 
+        # A map from outline to a mesh.
+        self.mesh_displayables = [ ]
+
         di = DrawInfo()
 
-        for o, color, _xo, _yo in self.outlines:
+        depth = len(self.outlines)
+        max_depth = depth - 1
+
+        for o, color, xo, yo in self.outlines:
+            depth -= 1
+
             key = (o, color)
 
             if key in self.textures:
@@ -857,8 +966,8 @@ class Layout(object):
 
             # Create the texture.
 
-            tw = int(sw + o)
-            th = int(sh + o)
+            tw = int(sw + o * 2)
+            th = int(sh + o * 2)
 
             # If not a multiple of 32, round up.
             tw = (tw | 0x1f) + 1 if (tw & 0x1f) else tw
@@ -906,8 +1015,26 @@ class Layout(object):
 
             self.textures[key] = tex
 
+            if self.textshaders:
+                for ts in self.textshaders:
+                    mr = self.create_mesh_displayable(o, tex, lines, xo, yo, depth, max_depth, ts)
+                    self.mesh_displayables.append((o, xo, yo, mr))
+
+        if self.textshaders:
+            extra_slow_time, extra_slow_duration, self.redraw, self.redraw_when_slow = renpy.text.shader.compute_times(self.textshaders)
+        else:
+            extra_slow_time = 0.0
+            extra_slow_duration = 0.0
+            self.redraw = None
+            self.redraw_when_slow = 0.0
+
         # Compute the max time for all lines, and the max max time.
-        self.max_time = textsupport.max_times(lines)
+        if not self.cps:
+            slow_duration = 0
+        else:
+            slow_duration = 1.0 / self.cps
+
+        self.max_time = textsupport.max_times(lines) + extra_slow_time + extra_slow_duration * slow_duration
 
         # Store the lines, so we have them for typeout.
         self.lines = lines
@@ -931,6 +1058,7 @@ class Layout(object):
                 renpy.display.to_log.write("     Available: (%d, %d) Laid-out: (%d, %d)", width, height, sw, sh)
                 renpy.display.to_log.write("     Text: %r", text.text)
 
+
     def make_alignment_grid(self, surf):
         w, h = surf.get_size()
 
@@ -994,7 +1122,7 @@ class Layout(object):
 
     def create_text_segments(self, text, ts, style):
         """
-        Creates one or more text segements, splitting out emoji. This
+        Creates one or more text segments, splitting out emoji. This
         will also use subsegment to handle font groups.
         """
 
@@ -1074,11 +1202,7 @@ class Layout(object):
         line = [ ]
 
         ts = TextSegment(None)
-
-        ts.cps = style.slow_cps
-        if ts.cps is None or ts.cps is True:
-            ts.cps = renpy.game.preferences.text_cps
-
+        ts.cps = self.cps
         ts.take_style(style, self)
 
         # The text segement stack.
@@ -1097,7 +1221,7 @@ class Layout(object):
 
         def fill_empty_line():
             for i in line:
-                if isinstance(i[0], (TextSegment, SpaceSegment, DisplayableSegment)):
+                if isinstance(i[0], TextSegment):
                     return
 
             line.extend(tss[-1].subsegment(u"\u200B")) # type: ignore
@@ -1120,8 +1244,7 @@ class Layout(object):
                 elif type == TEXT:
 
                     if (text_displayable.mask is not None):
-                        if text != u"\u200b":
-                            text = text_displayable.mask * len(text)
+                        text = text_displayable.mask * len(text)
 
                     line.extend(self.create_text_segments(text, tss[-1], style))
 
@@ -1169,7 +1292,7 @@ class Layout(object):
                     if len(value) < 1:
                         raise Exception("empty value supplied for tag %r" % tag)
 
-                    width = self.scale_int(int(value))
+                    width = float(value)
                     line.append((SpaceSegment(tss[-1], width=width), u""))
 
                 elif tag == "vspace":
@@ -1179,7 +1302,7 @@ class Layout(object):
 
                     # Duplicates from the newline tag.
 
-                    height = self.scale_int(int(value))
+                    height = float(value)
 
                     if line:
                         paragraphs.append(line)
@@ -1279,7 +1402,8 @@ class Layout(object):
                     if value[0] in "+-":
                         push().size += int(value)
                     elif value[0] == "*":
-                        push().size = int(float(value[1:]) * push().size)
+                        ts = push()
+                        ts.size = int(float(value[1:]) * ts.size)
                     else:
                         push().size = int(value)
 
@@ -1372,6 +1496,14 @@ class Layout(object):
                     ts.instance = value.lower()
                     ts.axis = None
 
+                elif tag == "shader":
+                    ts = push()
+
+                    if ts.shader is None:
+                        raise Exception("The shader tag was given, but text shaders are not in use. Consider adding \"define config.default_textshader = 'typewriter'\" to your game.")
+
+                    ts.shader = renpy.text.shader.get_textshader(value)
+
                 elif tag.startswith("axis:"):
                     ts = push()
                     if ts.axis:
@@ -1438,6 +1570,8 @@ class Layout(object):
             l.append((ts, s))
 
         rtl = (direction == RTL or direction == WRTL)
+        if rtl:
+            l.reverse()
 
         return l, rtl
 
@@ -1571,16 +1705,203 @@ class Layout(object):
 
         return rv
 
-    def redraw_typewriter(self, st):
+    def create_mesh_displayable(self, outline, tex, lines, xo, yo, depth, max_depth, ts):
         """
-        Return the time of the first glyph that should be shown after st.
+        Create a Displayable that will use a mesh to draw the text character-by-character.
+
+        `outline`
+            The size of the outline, in pixels.
+
+        `tex`
+            The texture to draw.
+
+        `lines`
+            A list of Line objects, representing the lines of text.
+
+        `xo`, `yo`
+            The x and y offsets of the text.
+
+        `depth`
+            The depth of the text. Depth 0 is closest to the screen, 1 is further away, and
+            so on.
+
+        `max_depth`
+            The maximum depth of any text.
+
+        `ts`
+            The text shader.
         """
 
-        if st >= self.max_time:
-            return None
+        first_shader = ts
+
+        for l in lines:
+            for g in l.glyphs:
+                first_shader = g.shader
+                break
+
+        tw, th = tex.get_size()
+
+        if lines:
+            last_line = lines[-1]
         else:
-            return 0
+            last_line = None
 
+        # The number of glyphs in the mesh.
+        n_glyphs = sum(len(l.glyphs) for l in lines)
+
+        mesh = renpy.gl2.gl2mesh2.Mesh2.text_mesh(n_glyphs + 2 * len(lines))
+
+        # The y coordinate of the top line.
+        top = 0
+
+        # The index of the last glyph to be shown.
+        last_index = 0
+
+        # The time of the last glyph to be shown.
+        last_time = 0.0
+
+        for line in lines:
+
+            # Line bottom is the y coordinate of the bottom line.
+            if line is not last_line:
+                bottom = min(outline + line.y + line.height + self.line_overlap_split, th)
+            else:
+                bottom = min(2 * outline + line.y + line.height, th)
+
+            if line.glyphs:
+                first_glyph = line.glyphs[0]
+                last_glyph = line.glyphs[-1]
+            else:
+                first_glyph = None
+                last_glyph = None
+
+            left = 0
+
+            if first_glyph:
+                right = first_glyph.x + self.add_left
+            else:
+                right = tw
+
+            # Generate a psuedo-glyph for the text to the left of the line.
+            # These pseudo-glyphs are used to make sure that outlines of lines above and below
+            # are displayed.
+
+            if outline:
+                if right > 0 and (ts == first_shader):
+
+                    cx = 0 + right / 2
+                    cy = outline + line.baseline
+
+                    mesh.add_glyph(
+                        tw, th,
+                        cx, cy,
+                        last_index,
+                        left, top, right, bottom,
+                        last_time, last_time,
+                        line.baseline, line.height - line.baseline,
+                        self.add_left, self.add_top,
+                    )
+
+            # Generate the actual glyphs.
+
+            for g in line.glyphs:
+
+                left = right
+
+                if g.time == -1:
+                    continue
+
+
+                # The x-coordinate of the right edge of the glyph.
+                if g is last_glyph:
+                    right = g.x + g.advance + outline * 2 + self.add_left + self.add_right
+                else:
+                    right = g.x + g.advance + outline + self.add_left
+
+                if left < 0:
+                    left = 0
+                if right > tw:
+                    right = tw
+
+                # The center coordinates of the glyph. These aren't the
+                # actual center, but the center of the baseline.
+                cx = g.x + g.advance / 2 + self.add_left
+                cy = outline + line.baseline
+
+                duration = g.duration
+                if g.duration < 0:
+                    duration = 0
+
+                if g.rtl:
+                    left_time = g.time
+                    right_time = g.time - duration
+                else:
+                    left_time = g.time - duration
+                    right_time = g.time
+
+
+                # Check that this is the right shader to use.
+                if (g.shader is ts) or (g.shader == ts):
+
+                    mesh.add_glyph(
+                        tw, th,
+                        cx, cy,
+                        g.index,
+                        left, top, right, bottom,
+                        left_time, right_time,
+                        g.ascent, g.descent,
+                        self.add_left, self.add_top,
+                    )
+
+                last_time = g.time
+                last_index = g.index
+
+            # Handle the empty space to the right of the last glyph.
+            if outline:
+
+                if right < tw and (ts == first_shader):
+
+                    left = right
+                    right = tw
+
+                    cx = left + right / 2
+                    cy = outline + line.baseline
+
+                    mesh.add_glyph(
+                        tw, th,
+                        cx, cy,
+                        last_index,
+                        left, top, right, bottom,
+                        last_time, last_time,
+                        line.baseline, line.height - line.baseline,
+                        self.add_left, self.add_top,
+                    )
+
+            top = bottom
+
+        main = (depth == 0)
+
+        # (name, value, is_displayable)
+        uniforms = [
+            ("u_text_depth", depth, False),
+            ("u_text_max_depth", max_depth, False),
+            ("u_text_outline", outline, False),
+            ("u_text_offset", (xo, yo), False),
+            ("u_text_main", 1.0 if main else 0.0, False),
+            ("u_text_to_virtual", 1 / self.oversample, False),
+            ("u_text_to_drawable", self.oversample, False),
+        ]
+
+        for k, v in ts.uniforms:
+            uniforms.append((k, v, isinstance(v, renpy.display.core.Displayable)))
+
+        return TextMeshDisplayable(
+            tw, th,
+            tex,
+            ts.shader,
+            mesh,
+            uniforms,
+        )
 
 # The maximum number of entries in the layout cache.
 LAYOUT_CACHE_SIZE = 50
@@ -1836,7 +2157,10 @@ class Text(renpy.display.displayable.Displayable):
         if self.locked:
             return False
 
-        self.language = renpy.game.preferences.language
+        if renpy.game.context().init_phase:
+            self.language = None
+        else:
+            self.language = renpy.game.preferences.language
 
         if self.tokenized:
 
@@ -2296,18 +2620,8 @@ class Text(renpy.display.displayable.Displayable):
         vw, vh = virtual_layout.size
         w, h = layout.size
 
-        # Get the list of blits we want to undertake.
-        if not self.slow:
-            blits = [ Blit(0, 0, w - layout.xborder, h - layout.yborder, left=True, right=True, top=True, bottom=True) ]
-            redraw = None
-        else:
-            # TODO: Make this changeable.
-            blits = layout.blits_typewriter(st)
-            redraw = layout.redraw_typewriter(st)
-
         # Blit text layers.
         rv = renpy.display.render.Render(vw, vh)
-        # rv = renpy.display.render.Render(*layout.unscale_pair(w, h))
 
         if renpy.config.draw_virtual_text_box:
             fill = renpy.display.render.Render(vw, vh)
@@ -2317,6 +2631,89 @@ class Text(renpy.display.displayable.Displayable):
 
             rv.blit(fill, (0, 0))
 
+        # Handle slow text ending.
+        if self.slow and st > layout.max_time:
+            self.slow = False
+            renpy.game.interface.timeout(0)
+
+        if layout.textshaders:
+            self.render_textshader(rv, layout, st, at)
+        else:
+            self.render_blits(rv, layout, st)
+
+        # Blit displayables.
+        if layout.displayable_blits:
+
+            self.displayable_offsets = [ ]
+
+            drend = renpy.display.render.Render(w, h)
+            drend.forward = layout.reverse
+            drend.reverse = layout.forward
+
+            for d, x, y, width, ascent, line_spacing, t in layout.displayable_blits:
+
+                if self.slow and t > st:
+                    continue
+
+                xo, yo = renpy.display.displayable.place(
+                    width,
+                    ascent,
+                    width,
+                    line_spacing,
+                    d.get_placement())
+
+                xo = x + xo + layout.xoffset
+                yo = y + yo + layout.yoffset
+
+                drend.absolute_blit(renders[d], (xo, yo))
+                self.displayable_offsets.append((d, xo, yo))
+
+            rv.blit(drend, (0, 0))
+
+        # Add in the focus areas.
+        for hyperlink, hx, hy, hw, hh, valid_st in layout.hyperlinks:
+
+            if st >= valid_st or not self.slow:
+
+                h_x, h_y = layout.unscale_pair(hx + layout.xoffset, hy + layout.yoffset)
+                h_w, h_h = layout.unscale_pair(hw, hh)
+
+                rv.add_focus(self, hyperlink, h_x, h_y, h_w, h_h)
+
+        # Figure out if we need to redraw.
+        if self.slow:
+            redraw = layout.redraw_when_slow
+        else:
+            redraw = layout.redraw
+
+        if redraw is not None:
+            renpy.display.render.redraw(self, max(redraw, 0))
+
+        rv.forward = layout.forward
+        rv.reverse = layout.reverse
+
+        if self.style.vertical:
+            vrv = renpy.display.render.Render(rv.height, rv.width)
+            vrv.forward = VERT_FORWARD
+            vrv.reverse = VERT_REVERSE
+            vrv.blit(rv, (rv.height, 0))
+            rv = vrv
+
+        if layout.pixel_perfect:
+            rv.properties = { "pixel_perfect" : True }
+
+        return rv
+
+    def render_blits(self, render, layout, st):
+
+        w, h = layout.size
+
+        # Get the list of blits we want to undertake.
+        if not self.slow:
+            blits = [ Blit(0, 0, w - layout.xborder, h - layout.yborder, left=True, right=True, top=True, bottom=True) ]
+        else:
+            blits = layout.blits_typewriter(st)
+
         for o, color, xo, yo in layout.outlines:
             tex = layout.textures[o, color]
 
@@ -2369,73 +2766,37 @@ class Text(renpy.display.displayable.Displayable):
                     b_y += layout.add_top
 
                 # Blit.
-                rv.absolute_blit(
+                render.absolute_blit(
                     tex.subsurface((b_x, b_y, b_w, b_h)),
                     layout.unscale_pair(b_x + xo + layout.xoffset - o - layout.add_left,
                                         b_y + yo + layout.yoffset - o - layout.add_top)
                     )
 
-        # Blit displayables.
-        if layout.displayable_blits:
+    def render_textshader(self, render, layout, st, at):
 
-            self.displayable_offsets = [ ]
+        slow_time = layout.max_time
 
-            drend = renpy.display.render.Render(w, h)
-            drend.forward = layout.reverse
-            drend.reverse = layout.forward
+        if self.slow and layout.cps:
+            slow_time = min(slow_time, st)
 
-            for d, x, y, width, ascent, line_spacing, t in layout.displayable_blits:
+        render.add_uniform("u_text_slow_time", slow_time)
 
-                if self.slow and t > st:
-                    continue
+        if layout.cps:
+            render.add_uniform("u_text_slow_duration", 1.0 / layout.cps)
+        else:
+            render.add_uniform("u_text_slow_duration", 0.0)
 
-                xo, yo = renpy.display.displayable.place(
-                    width,
-                    ascent,
-                    width,
-                    line_spacing,
-                    d.get_placement())
+        for o, xo, yo, mesh_displayable in layout.mesh_displayables:
+            mesh_render = renpy.display.render.render(mesh_displayable, layout.width, layout.height, st, at)
 
-                xo = x + xo + layout.xoffset
-                yo = y + yo + layout.yoffset
-
-                drend.absolute_blit(renders[d], (xo, yo))
-                self.displayable_offsets.append((d, xo, yo))
-
-            rv.blit(drend, (0, 0))
-
-        # Add in the focus areas.
-        for hyperlink, hx, hy, hw, hh, valid_st in layout.hyperlinks:
-
-            if st >= valid_st or not self.slow:
-
-                h_x, h_y = layout.unscale_pair(hx + layout.xoffset, hy + layout.yoffset)
-                h_w, h_h = layout.unscale_pair(hw, hh)
-
-                rv.add_focus(self, hyperlink, h_x, h_y, h_w, h_h)
-
-        # Figure out if we need to redraw or call slow_done.
-        if self.slow:
-            if redraw is not None:
-                renpy.display.render.redraw(self, max(redraw, 0))
-            else:
-                self.slow = False
-                renpy.game.interface.timeout(0)
-
-        rv.forward = layout.forward
-        rv.reverse = layout.reverse
-
-        if self.style.vertical:
-            vrv = renpy.display.render.Render(rv.height, rv.width)
-            vrv.forward = VERT_FORWARD
-            vrv.reverse = VERT_REVERSE
-            vrv.blit(rv, (rv.height, 0))
-            rv = vrv
-
-        if layout.pixel_perfect:
-            rv.properties = { "pixel_perfect" : True }
-
-        return rv
+            render.absolute_blit(
+                mesh_render,
+                layout.unscale_pair(
+                    xo + layout.xoffset - o,
+                    yo + layout.yoffset - o),
+                focus=False,
+                main=False,
+            )
 
     def tokenize(self, text):
         """
